@@ -2,24 +2,21 @@
  * Copyright (c) 1998-2000 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ *
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
+ *
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
- * 
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
+ *
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -226,11 +223,16 @@ void IOAudioClientBufferSet::watchdogTimerFired(IOAudioClientBufferSet *clientBu
 #endif
 
 		userClient = clientBufferSet->userClient;
-		userClient->retain();
-		userClient->lockBuffers();
-
-		if(clientBufferSet->timerPending != false) {
-			userClient->performWatchdogOutput(clientBufferSet, generationCount);
+		if (userClient) {
+			userClient->retain();
+			userClient->lockBuffers();
+	
+			if(clientBufferSet->timerPending != false) {
+				userClient->performWatchdogOutput(clientBufferSet, generationCount);
+			}
+	
+			userClient->unlockBuffers();
+			userClient->release();
 		}
 
 		// If there's no timer pending once we attempt to do the watchdog I/O
@@ -238,9 +240,6 @@ void IOAudioClientBufferSet::watchdogTimerFired(IOAudioClientBufferSet *clientBu
 		if (!clientBufferSet->timerPending) {
 			clientBufferSet->release();
 		}
-
-		userClient->unlockBuffers();
-		userClient->release();
 	} else {
 		IOLog ("IOAudioClientBufferSet::watchdogTimerFired assert (clientBufferSet == NULL) failed\n");
 	}
@@ -584,6 +583,9 @@ void IOAudioEngineUserClient::freeClientBufferSetList()
     while (clientBufferSetList) {
         IOAudioClientBufferSet *nextSet;
         
+		// Move call up here to fix 3472373
+        clientBufferSetList->cancelWatchdogTimer();
+
         while (clientBufferSetList->outputBufferList) {
             IOAudioClientBuffer *next = clientBufferSetList->outputBufferList->next;
             
@@ -602,7 +604,6 @@ void IOAudioEngineUserClient::freeClientBufferSetList()
         
         nextSet = clientBufferSetList->next;
         
-        clientBufferSetList->cancelWatchdogTimer();
         clientBufferSetList->release();
         
         clientBufferSetList = nextSet;
